#
# EPrints Services - REF Package
#
# Version: 1.1
#


# REF2 Report

$c->{plugins}->{"Screen::REF::Report::REF2::View"}->{params}->{validate_selection} = sub {
	my( $user, $selection, $eprint, $ctx ) = @_;

	my $session = $user->{session};

	my @problems;

	if( $selection->is_set( "details" ) )
	{
		my @words = split /\s+/, $selection->value( "details" );
		if( @words > 300 )
		{
			push @problems, $session->html_phrase( "ref:validate:word_limit",
				field => $selection->{dataset}->field( "details" )->render_name( $session ),
				length => $session->make_text( scalar @words ),
				limit => $session->make_text( 300 ),
			);
		}
	}
	if( $selection->is_set( "abstract" ) )
	{
		my @words = split /\s+/, $selection->value( "abstract" );
		if( @words > 100 )
		{
			push @problems, $session->html_phrase( "ref:validate:word_limit",
				field => $selection->{dataset}->field( "abstract" )->render_name( $session ),
				length => $session->make_text( scalar @words ),
				limit => $session->make_text( 100 ),
			);
		}
	}

	if( $selection->is_set( 'reserve' ) )
	{
		# can't have chosen a reserved output if the current selection is not double-weighted
		if( !$selection->is_set( 'weight' ) || $selection->get_value( 'weight' ) ne 'double' )
		{
			push @problems, $session->html_phrase( "ref:validate:wrong_reserve" );
		}
	
		# a selection can't be double-weighted and a reserved for itself
		if( $selection->get_value( 'reserve' ) eq $selection->get_id )
		{
			push @problems, $session->html_phrase( "ref:validate:self_reserve" );
		}

	}
	else
	{
		# if the output is double weighted then it must reference a reserved output
		if( $selection->is_set( 'weight' ) && $selection->value( 'weight' ) eq 'double' )
		{
			push @problems, $session->html_phrase( "ref:validate:missing_field",
					fieldname => $session->html_phrase( 'ref_selection_fieldname_reserve' )
			);

		}
	}
	
	# if the output is double-weighted then must provide a statement
	if( $selection->is_set( 'weight' ) && $selection->value( 'weight' ) eq 'double' && !$selection->is_set( 'weight_text' ) ) 
	{
                push @problems, $session->html_phrase( "ref:validate:missing_field",
                        fieldname => $session->html_phrase( 'ref_selection_fieldname_weight_text' )
                );	
	}

	# if the output is non-english, then must provide an English abstract
	if( $selection->is_set( 'non_english' ) && $selection->value( 'non_english' ) eq 'TRUE' && !$selection->is_set( 'abstract' ) )
	{
                push @problems, $session->html_phrase( "ref:validate:missing_field",
                        fieldname => $session->html_phrase( 'ref_selection_fieldname_abstract' )
                );
	}

	# if the output has conflicts of interest then must provide the list of the conflicted panel members
	if( $selection->is_set( 'has_conflicts' ) && $selection->value( 'has_conflicts' ) eq 'TRUE' && !$selection->is_set( 'conflicted_members' ) )
	{
                push @problems, $session->html_phrase( "ref:validate:missing_field",
                        fieldname => $session->html_phrase( 'ref_selection_fieldname_conflicted_members' )
                );
	}

	# if the output is non-english, then must provide an English abstract
	if( $selection->is_set( 'is_xref' ) && $selection->value( 'is_xref' ) eq 'TRUE' && !$selection->is_set( 'xref' ) )
	{
                push @problems, $session->html_phrase( "ref:validate:missing_field",
                        fieldname => $session->html_phrase( 'ref_selection_fieldname_xref' )
                );
	}

	my $plugin = $session->plugin( "Export::REF2014" );
	if( defined $plugin )
	{
		$plugin->dataobj_to_refdata( $selection,
			eprint => $eprint,
			user => $user,
			problems => \@problems,
		);
	}

	my $year = $eprint->value( "date" );
	$year = "" if !defined $year;
	$year = substr($year,0,4);

	if( !$year || $year < 2008 || $year > 2013 )
	{
		push @problems, $session->html_phrase( "ref:validate:year",
			year => $session->make_text( $year ),
		);
	}

	return @problems;
};

# REF1 Report

$c->{plugins}->{"Screen::REF::Report::REF1::View"}->{params}->{validate_user} = sub {
	my( $user, $ctx ) = @_;

	my $session = $user->{session};

	my @problems;

	my $type = $user->value( 'ref_category' );

	unless( defined $type )
	{
		push @problems, $session->html_phrase( "ref:validate_user:no_category");
		return @problems;	# can't go further, we must know the category
	}

	if( $type eq 'B' || $type eq 'D' )
	{
		push @problems, $session->html_phrase( "ref:validate_user:wrong_category");
		return @problems;	# can't go further, we must know the category
	}

# extra checks for Category A Staff:

# for HEFCW institutions, whether the user is a research fellow or not
# whether the user is an ECR (early career researcher)
# ref_start_date # if between 1 Jan 2008 and 31 October 2013
# if user on a fixed-term contract, secondment, or period of unpaid leaves, start and end dates of the contract, 2ndment or unpaid leaves

	# internal_id as in the Institution's unique id (employee id or else)
	my $mf = [ 'internal_id', 'name_family', 'name_given' ];

	if( $type eq 'A' )
	{
		push @$mf, ( 'hesa', 'dob', 'fte' );	#, 'ref_start_date' );
	}

	my $ds = $user->get_dataset;
	foreach( @$mf )
	{
		next unless( $ds->has_field( "$_" ) );

		unless( $user->is_set( "$_" ) )
		{
			push @problems, $session->html_phrase( "ref:validate_user:missing:$_" );
		}

		# FTE must be >= 0.2
		if( "$_" eq 'fte' && $user->get_value( "$_") < 0.2 )
		{
			push @problems, $session->html_phrase( "ref:validate_user:low_fte" );
		}

	}

	return @problems;
};


=pod 

TO COMPLETE

# Reference: http://www.ref.ac.uk/subguide/submissionsystemdatarequirements/
# http://www.ref.ac.uk/media/ref/content/subguide/11.Output_requirements.XLSX
sub ref_required_fields
{
	my( $type ) = @_;

	my $req_fields = {

	# a question mark '?' after a field name denotes whether it's optional

		# A - Authored book
		A => [ "title", "publisher", "isbn", "doi?", "url?"  ],
		# B - Edited book
		B => [ "title", "publisher", "isbn", "doi?", "url?" ],
		# C - Chapter in book
		C => [ "title", "book_title", "isbn", "doi?", "url?"  ],
# TODO "article number" ? issue number?
		# D - Journal article	
		D => [ "title", "publication", "id_number", "pagerange", "volume", "issn", "doi", "url?" ],
# TODO issue number?
		# E - Conference contribution
		E => [ "title", "event_title", "pages", "volume", "number", "issn", "doi?", "url?"  ],
		# F - Patent
		F => [ "title", "id_number", "url?" ],
		# G - Software
		G => [ "title", "publisher", "output_media", "url?" ],
# url => official_url?
		# H - Website
		H => [ "title", "output_media", "url" ],
		# I - Performance
		I => [ "title" "event_location", "url" ],
		# J - Composition
		J => [],
		# K - Design
		K => [],
		# L - Artefact
		L => [],
		# M - Exhibition
		M => [],
		# N - Research report for external body
		N => [],
		# O - Confidential report
		O => [],
		# P - Devices and products
		P => [],
		# Q - Digital or visual media
		Q => [],
		# R - Scholarly edition
		R => [],
		# S - Research datasets and databases
		S => [],
		# T - Other
		T => [],
		# U - Working paper
		U => [],
	
	};

	return $req_fields->{$type};
}


=cut


1;

