package EPrints::Plugin::Screen::REF::Benchmark::New;

# "render" method shows a warning (to make sure the admins know what they're doing when pressing 'create')
# "action_create" method actually creates a new (empty benchmark)"

use EPrints::Plugin::Screen::REF;
@ISA = ( 'EPrints::Plugin::Screen::REF' );

use strict;

sub new
{
	my( $class, %params ) = @_;

	my $self = $class->SUPER::new(%params);

	$self->{actions} = [qw/ create copy cancel /];

	$self->{appears} = [
		{
			place => "ref_benchmark_create",
			action => "copy",
			position => 200,
		},
		{
			place => "ref_benchmark_create",
			action => "create",
			position => 200,
		},
		{
			place => "ref_benchmark_create",
			action => "cancel",
			position => 100,
		},
		{
			place => "dataobj_tools",
			position => 100,
		},
	];

	return $self;
}

sub can_be_viewed 
{ 
	my( $self ) = @_;

        my $ds = $self->{processor}->{dataset}; # set by Screen::Listing

	return $self->allow( 'ref_benchmark/create_new' ) if( defined $ds && ($ds->base_id eq 'ref_benchmark' || $ds->base_id eq 'ref_selection' ));
	return 0;
}

sub properties_from
{
	my( $self ) = @_;
	
	my $processor = $self->{processor};
	my $session = $self->{session};

	my $datasetid = "ref_benchmark";

	my $dataset = $session->dataset( $datasetid );
	if( !defined $dataset )
	{
		$processor->{screenid} = "Error";
		$processor->add_message( "error", $session->html_phrase(
			"lib/history:no_such_item",
			datasetid=>$session->make_text( $datasetid ),
			objectid=>$session->make_text( "" ) ) );
		return;
	}

	$processor->{"dataset"} = $dataset;

	$self->SUPER::properties_from;
}

sub allow_copy
{
	my ( $self ) = @_;

	return $self->can_be_viewed();
}

sub allow_create
{
	my ( $self ) = @_;

	return $self->can_be_viewed();
}

sub allow_cancel
{
	my ( $self ) = @_;

	return $self->can_be_viewed();
}

sub action_cancel
{
	my( $self ) = @_;

	# redirect somewhere sensible
	$self->{processor}->{redirect} = $self->{session}->config( 'userhome' )."?screen=Listing&dataset=ref_benchmark";
}	

sub action_copy
{
	my( $self ) = @_;

	$self->{processor}->{screenid} = 'REF::Benchmark::Copy';

	return;
}

sub action_create
{
	my( $self ) = @_;

	my $ds = $self->{processor}->{dataset};

	my $epdata = {};

	if( defined $ds->field( "userid" ) )
	{
		my $user = $self->{session}->current_user;
		$epdata->{userid} = $user->id;
	}

	$self->{processor}->{dataobj} = $ds->create_dataobj( $epdata );

	if( !defined $self->{processor}->{dataobj} )
	{
		my $db_error = $self->{session}->get_database->error;
		$self->{processor}->{session}->get_repository->log( "Database Error: $db_error" );
		$self->{processor}->add_message( 
			"error",
			$self->html_phrase( "db_error" ) );
		return;
	}

	$self->{processor}->{screenid} = "Workflow::Edit";
}

sub render
{
	my( $self ) = @_;

	my $session = $self->{session};

	my $chunk = $session->make_doc_fragment;

	$chunk->appendChild( $self->html_phrase( 'create_warning') );

	$chunk->appendChild( $self->render_action_list_bar( "ref_benchmark_create" ) );

	return $chunk;
}


1;
